require 'monitor'

class SimpleQueue
  def initialize
    @monitor = Monitor.new
    @enqueued = @monitor.new_cond
    @queue = []
  end

  def enqueue(value)
    @monitor.synchronize {
      @queue.unshift(value)
      @enqueued.signal
    }
    self
  end

  def dequeue()
    @monitor.synchronize {
      @enqueued.wait_while { @queue.empty? }
      return @queue.pop
    }
  end  
end

if $0 == __FILE__
  eval DATA.read, nil, $0, __LINE__+4
end

__END__

queue = SimpleQueue.new

producers = (1..3).map { |i|
  Thread.new {
    puts "producer #{i} starts"
    sleep rand(5)
    letter = (10 + rand(26)).to_s(36).upcase
    puts "producer #{i} is enqueuing the letter #{letter}"
    queue.enqueue(letter)
  }
}

consumers = (1..3).map { |i|
  Thread.new {
    puts "consumer #{i} starts"
    letter = queue.dequeue
    puts "consumer #{i} got the letter #{letter}"
  }
}

(producers + consumers).each { |th|
  th.join
}
